@@ -6,7 +6,7 @@
 ;;;; Purpose:  Function definitions for UFFI interface to ODBC
 ;;;; Author:   Kevin M. Rosenberg
 ;;;;
-;;;; $Id: odbc-ff-interface.lisp 10078 2004-10-01 17:24:52Z kevin $
+;;;; $Id: odbc-ff-interface.lisp 10845 2005-11-26 16:08:12Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2004 by Kevin M. Rosenberg
 ;;;; and Copyright (C) Paul Meurer 1999 - 2001. All rights reserved.
@@ -61,7 +61,7 @@
 (def-function "SQLDriverConnect"
     ((hdbc sql-handle)          ; HDBC        hdbc
      (hwnd sql-handle)          ; SQLHWND     hwnd
-     (*szConnStrIn string-ptr)  ; UCHAR  FAR *szConnStrIn
+     (*szConnStrIn :cstring)    ; UCHAR  FAR *szConnStrIn
      (cbConnStrIn :short)       ; SWORD       cbConnStrIn
      (*szConnStrOut string-ptr) ; UCHAR  FAR *szConnStrOut
      (cbConnStrOutMax :short)   ; SWORD       cbConnStrOutMax
