@@ -7,7 +7,7 @@
 ;;;; Authors:  Kevin M. Rosenberg based on original code by Pierre R. Mai 
 ;;;; Created:  Feb 2002
 ;;;;
-;;;; $Id: postgresql-socket-api.lisp 10490 2005-04-28 14:35:56Z kevin $
+;;;; $Id: postgresql-socket-api.lisp 10868 2006-01-16 21:46:13Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2004 by Kevin M. Rosenberg
 ;;;; and Copyright (c) 1999-2001 by Pierre R. Mai
@@ -334,10 +334,14 @@ socket interface"
   (etypecase host
     (pathname
      ;; Directory to unix-domain socket
-     (sb-bsd-sockets:socket-connect
-      (namestring
-       (make-pathname :name ".s.PGSQL" :type (princ-to-string port)
-		      :defaults host))))
+     (let ((sock (make-instance 'sb-bsd-sockets:local-socket
+				:type :stream)))
+       (sb-bsd-sockets:socket-connect
+        sock
+        (namestring
+         (make-pathname :name ".s.PGSQL" :type (princ-to-string port)
+                        :defaults host)))
+       sock))
     (string
      (let ((sock (make-instance 'sb-bsd-sockets:inet-socket
 				:type :stream
