@@ -6,7 +6,7 @@
 ;;;; Purpose:       High-level PostgreSQL interface using UFFI
 ;;;; Date Started:  Feb 2002
 ;;;;
-;;;; $Id: postgresql-sql.lisp 10077 2004-10-01 17:02:20Z kevin $
+;;;; $Id: postgresql-sql.lisp 10742 2005-09-18 00:13:11Z kevin $
 ;;;;
 ;;;; CLSQL users are granted the rights to distribute and use this software
 ;;;; as governed by the terms of the Lisp Lesser GNU Public License
@@ -412,7 +412,7 @@
 
 (defmethod database-probe (connection-spec (type (eql :postgresql)))
   (when (find (second connection-spec) (database-list connection-spec type)
-	      :key #'car :test #'string-equal)
+              :test #'string-equal)
     t))
 
 
