@@ -1,7 +1,7 @@
 ;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
 ;;;; *************************************************************************
 ;;;;
-;;;; $Id: generic-postgresql.lisp 10156 2004-11-12 15:11:39Z kevin $
+;;;; $Id: generic-postgresql.lisp 10868 2006-01-16 21:46:13Z kevin $
 ;;;;
 ;;;; Generic postgresql layer, used by db-postgresql and db-postgresql-socket
 ;;;;
@@ -49,15 +49,25 @@
   (declare (ignore args database))
   "TIMESTAMP WITHOUT TIME ZONE")
 
+(defmethod database-get-type-specifier ((type (eql 'number)) args database
+                                        (db-type (eql :postgresql)))
+  (declare (ignore database))
+  (cond
+   ((and (consp args) (= (length args) 2))
+    (format nil "NUMERIC(~D,~D)" (first args) (second args)))
+   ((and (consp args) (= (length args) 1))
+    (format nil "NUMERIC(~D)" (first args)))
+   (t
+    "NUMERIC")))
 
 ;;; Backend functions
 
 (defun owner-clause (owner)
-  (cond 
+  (cond
    ((stringp owner)
     (format
      nil
-     " AND (relowner=(SELECT usesysid FROM pg_user WHERE (usename='~A')))" 
+     " AND (relowner=(SELECT usesysid FROM pg_user WHERE (usename='~A')))"
      owner))
    ((null owner)
     (format nil " AND (NOT (relowner=1))"))
@@ -75,11 +85,11 @@
 (defmethod database-list-tables ((database generic-postgresql-database)
                                  &key (owner nil))
   (database-list-objects-of-type database "r" owner))
-  
+
 (defmethod database-list-views ((database generic-postgresql-database)
                                 &key (owner nil))
   (database-list-objects-of-type database "v" owner))
-  
+
 (defmethod database-list-indexes ((database generic-postgresql-database)
                                   &key (owner nil))
   (database-list-objects-of-type database "i" owner))
@@ -89,7 +99,7 @@
 					&key (owner nil))
   (let ((indexrelids
 	 (database-query
-	  (format 
+	  (format
 	   nil
 	   "select indexrelid from pg_index where indrelid=(select relfilenode from pg_class where relname='~A'~A)"
 	   (string-downcase table)
@@ -97,7 +107,7 @@
 	  database :auto nil))
 	(result nil))
     (dolist (indexrelid indexrelids (nreverse result))
-      (push 
+      (push
        (caar (database-query
 	      (format nil "select relname from pg_class where relfilenode='~A'"
 		      (car indexrelid))
@@ -127,7 +137,7 @@
                                                "oid"
                                                "ctid"
                                                ;; kmr -- added tableoid
-                                               "tableoid") :test #'equal)) 
+                                               "tableoid") :test #'equal))
                    result))))
 
 (defmethod database-attribute-type (attribute (table string)
@@ -140,15 +150,29 @@
 			   (owner-clause owner))
 		   database nil nil))))
     (when row
-      (values
-       (ensure-keyword (first row))
-       (if (string= "-1" (second row))
-	   (- (parse-integer (third row) :junk-allowed t) 4)
-	 (parse-integer (second row)))
-       nil
-       (if (string-equal "f" (fourth row))
-	   1
-	 0)))))
+      (destructuring-bind (typname attlen atttypmod attnull) row
+
+        (setf attlen (parse-integer attlen :junk-allowed t)
+              atttypmod (parse-integer atttypmod :junk-allowed t))
+
+        (let ((coltype (ensure-keyword typname))
+              (colnull (if (string-equal "f" attnull) 1 0))
+              collen
+              colprec)
+           (setf (values collen colprec)
+                 (case coltype
+                   ((:numeric :decimal)
+                    (if (= -1 atttypmod)
+                        (values nil nil)
+                        (values (ash (- atttypmod 4) -16)
+                                (boole boole-and (- atttypmod 4) #xffff))))
+                   (otherwise
+                    (values
+                     (cond ((and (= -1 attlen) (= -1 atttypmod)) nil)
+                           ((= -1 attlen) (- atttypmod 4))
+                           (t attlen))
+                     nil))))
+           (values coltype collen colprec colnull))))))
 
 (defmethod database-create-sequence (sequence-name
 				     (database generic-postgresql-database))
@@ -174,7 +198,7 @@
       (format nil "SELECT SETVAL ('~A', ~A)" name position)
       database nil nil)))))
 
-(defmethod database-sequence-next (sequence-name 
+(defmethod database-sequence-next (sequence-name
 				   (database generic-postgresql-database))
   (values
    (parse-integer
@@ -199,7 +223,7 @@
       (unwind-protect
 	   (progn
 	     (setf (slot-value database 'clsql-sys::state) :open)
-	     (mapcar #'car (database-query "select datname from pg_database" 
+	     (mapcar #'car (database-query "select datname from pg_database"
 					   database nil nil)))
 	(progn
 	  (database-disconnect database)
@@ -211,13 +235,13 @@
 (defmethod database-list (connection-spec (type (eql :postgresql-socket)))
   (postgresql-database-list connection-spec type))
 
-#+nil 
+#+nil
 (defmethod database-describe-table ((database generic-postgresql-database) table)
   ;; MTP: LIST-ATTRIBUTE-TYPES currently executes separate queries for
   ;; each attribute. It would be more efficient to have a single SQL
   ;; query return the type data for all attributes. This code is
   ;; retained as an example of how to do this for PostgreSQL.
-  (database-query 
+  (database-query
    (format nil "select a.attname, t.typname
                                from pg_class c, pg_attribute a, pg_type t
                                where c.relname = '~a'
@@ -249,8 +273,8 @@
     ((in type :float :double :number) "NUMERIC")
     ((and (consp type) (in (car type) :char :varchar)) "VARCHAR")
     (t
-     (error 'sql-user-error 
-	    :message 
+     (error 'sql-user-error
+	    :message
 	    (format nil "Unknown clsql type ~A." type)))))
 
 (defun prepared-sql-to-postgresql-sql (sql)
@@ -270,7 +294,7 @@
 	  (setq in-str (not in-str))
 	  (write-char c out))
 	 ((and (char= c #\?) (not in-str))
-	  (write-char #\$ out) 
+	  (write-char #\$ out)
 	  (write-string (write-to-string (incf param)) out))
 	 (t
 	  (write-char c out)))))))
@@ -291,7 +315,7 @@
 		   :bindings (make-list (length types)))))
 
 (defmethod database-bind-parameter ((stmt postgresql-stmt) position value)
-  (setf (nth (1- position) (bindings stmt)) value)) 
+  (setf (nth (1- position) (bindings stmt)) value))
 
 (defun binding-to-param (binding)
   (typecase binding
