@@ -3,7 +3,7 @@
 ;;;; File:    test-init.lisp
 ;;;; Authors: Marcus Pearce <m.t.pearce@city.ac.uk>, Kevin Rosenberg
 ;;;; Created: 30/03/2004
-;;;; Updated: $Id: test-init.lisp 10281 2005-01-26 18:59:10Z kevin $
+;;;; Updated: $Id: test-init.lisp 10845 2005-11-26 16:08:12Z kevin $
 ;;;;
 ;;;; Initialisation utilities for running regression tests on CLSQL. 
 ;;;;
@@ -614,6 +614,15 @@
                              '(:postgresql :oracle)))
 		(clsql-sys:in test :fddl/owner/1))
            (push (cons test "table ownership not supported") skip-tests))
+          ((and (null (clsql-sys:db-type-has-intersect? db-underlying-type))
+		(clsql-sys:in test :fdml/query/7))
+	   (push (cons test "intersect not supported") skip-tests))
+          ((and (null (clsql-sys:db-type-has-except? db-underlying-type))
+		(clsql-sys:in test :fdml/query/8))
+	   (push (cons test "except not supported") skip-tests))
+          ((and (eq *test-database-underlying-type* :mssql)
+                (clsql-sys:in test :fdml/select/9))
+           (push (cons test "mssql uses integer math for AVG") skip-tests))
 	  (t
 	   (push test-form test-forms)))))
       (values (nreverse test-forms) (nreverse skip-tests))))
