@@ -7,7 +7,7 @@
 ;;;; Author:   Kevin M. Rosenberg
 ;;;; Created:  Mar 2002
 ;;;;
-;;;; $Id: clsql-uffi-loader.lisp 10612 2005-07-06 00:50:20Z kevin $
+;;;; $Id: clsql-uffi-loader.lisp 10667 2005-08-22 20:24:22Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2004 by Kevin M. Rosenberg
 ;;;;
@@ -37,8 +37,9 @@
                                       (warn "~A" c)
                                       nil))))
       (when errorp
-        (error "Couldn't load foreign librar~@P ~{~S~^, ~}."
-               (length filenames) filenames))))
+	(error "Couldn't load foreign librar~@P ~{~S~^, ~}. (searched ~S)"
+               (length filenames) filenames
+	       'clsql:*foreign-library-search-paths*))))
 
 (defvar *clsql-uffi-library-filenames*
     (list #+(or 64bit x86-64) "clsql_uffi64"
